//
//  FBullCowGame.cpp
//  BullCowGame
//
//  Created by Brett Mayes on 20/12/2019.
//  Copyright © 2019 Brett Mayes. All rights reserved.
//
#pragma once
#include "FBullCowGame.hpp"
#include <map>
#define TMap std::map

//Make code consistent with Unreal coding practice
using int32 = int;
using FString = std::string;

FBullCowGame::FBullCowGame() { Reset(); }


int32 FBullCowGame::GetCurrentTry() const { return MyCurrentTry; }
int32 FBullCowGame::GetHiddenWordLength() const { return (int32)MyHiddenWord.length(); }
bool FBullCowGame::IsGameWon() const { return bIsWon; }

void FBullCowGame::GetWord(FString LEN){
    FString ThreeLetters[] = {"hit","row","fix","leg","car","due","act","war","set","bin"};
    FString FourLetters[] = {"have","slab","rise","wage","pack","site","cord","real","heir","dome"};
    FString FiveLetters[] = {"salon","rider","south","angle","noise","laser","fling","habit","faint","witch"};
    FString SixLetters[] = {"action","belong","period","friend","aspect","reform","closed","safety","ethnic","insert"};
    FString SevenLetters[] = {"mislead","nuclear","density","convert","confuse","crystal","privacy","realism","freight","variety"};
    TMap<FString,FString*> Words {{"3",ThreeLetters}, {"4",FourLetters}, {"5",FiveLetters}, {"6",SixLetters}, {"7",SevenLetters}};
    int32 NUM = std::rand() % 9 + 1;
    MyHiddenWord = Words[LEN][NUM];
}

void FBullCowGame::SetDifficulty(FString UserDifficulty) {
    TMap<FString,int32> Level {{"easy",4}, {"normal",0}, {"hard",-4}};
    Difficulty = Level[UserDifficulty];
}

int32 FBullCowGame::GetMaxTries() const
{
    TMap<int32,int32> WordLengthToMaxTries {{3,4}, {4,7}, {5,10}, {6,15}, {7,20}};
    return std::max(WordLengthToMaxTries[GetHiddenWordLength()] + Difficulty,4);
}

void FBullCowGame::Reset() { 
    MyCurrentTry = 1;
    bIsWon = false;
    return;
}

EWordStatus FBullCowGame::CheckGuessValidity(FString Guess) {
    if(!IsIsogram(Guess))
    {
        return EWordStatus::Not_Isogram;
    }
    else if(!IsLowerCase(Guess))
    {
        return EWordStatus::Not_Lowercase;
    }
    else if(Guess.length() != GetHiddenWordLength())
    {
        return EWordStatus::Wrong_Length;
    }
    else
    {
    return EWordStatus::OK;
    }
}



//receives guess, increments turn number, returns count
FBullCowCount FBullCowGame::SubmitValidGuess(FString Guess) {
    
    //increment turn number
    MyCurrentTry++;
    
    //setup return variable
    FBullCowCount BullCowCount;
    int32 HiddenWordLength = GetHiddenWordLength();
    for(int32 i = 0; i < HiddenWordLength; i++)
    {
        for(int32 j = 0; j < HiddenWordLength; j++)
        {
            if(Guess[i] == MyHiddenWord[j])
            {
                if(i==j)
                {
                    BullCowCount.Bulls++;
                }
                else
                {
                    BullCowCount.Cows++;
                }
            }
        }
    }
    if(BullCowCount.Bulls == HiddenWordLength)
    {
        bIsWon = true;
    }
    return BullCowCount;
}


bool FBullCowGame::IsIsogram(FString Word) const
{
    if(Word.length() <= 1){ return true; }
    
    TMap<char, bool> LetterSeen;
    for(auto Letter : Word)
    {
        Letter = tolower(Letter);
        if(LetterSeen[Letter]){
            return false;
        } else {
            LetterSeen[Letter] = true;
        }
    }
    
    return true;
}

bool FBullCowGame::IsLowerCase(FString Word) const
{
    if(Word.length() < 1){ return true; }

    for(auto Letter : Word)
    {
        if(!islower(Letter))
        {
            return false;
        }
    }
    return true;
}
