//
//  main.cpp
//  BullCowGame
//
//  Created by Brett Mayes on 18/12/2019.
//  Copyright © 2019 Brett Mayes. All rights reserved.
//

/*This is the main executable of the Bulls And Cows game, it is responsible for all user interaction and uses the FBullCowGame class*/
#pragma once
#include <iostream>
#include <string>
#include "FBullCowGame.hpp"

//Make code more consistent with Unreal coding practice
using FText = std::string;
using int32 = int;

//Function prototypes as outside a class
void PrintIntro();
FText GetValidGuess();
void PlayGame();
bool AskToPlayAgain();
void PrintGameSummary();
void InputWordLength();
bool PlayAgainSameWord();
void InputDifficulty();

//instantiate new game, which is re-used across all plays
FBullCowGame BCGame;

// Main function, runs game until user quits
int main(int32 argc, const char * argv[]) {
    do
    {
    PrintIntro();
    InputWordLength();
    InputDifficulty();
    do
    {
        PlayGame();
    }
    while(AskToPlayAgain());
    }while(PlayAgainSameWord());
    std::cout << std::endl;
    std::cout << " ,----.                                                               " << std::endl;
    std::cout << "'  .-./    ,--,--.,--,--,--. ,---.      ,---.,--.  ,--.,---. ,--.--.  " << std::endl;
    std::cout << "|  | .---.' ,-.  ||        || .-. :    | .-. |\\  `'  /| .-. :|  .--'  " << std::endl;
    std::cout << "'  '--'  |\\ '-'  ||  |  |  |\\   --.    ' '-' ' \\    / \\   --.|  |     " << std::endl;
    std::cout << " `------'  `--`--'`--`--`--' `----'     `---'   `--'   `----'`--'     " << std::endl;
    return 0;
}

void PrintIntro()
{
    // introduce the game
    
    //ASCII ART
    std::cout << " ,-----.          ,--.,--.                              ,--.     ,-----. " << std::endl;
    std::cout << " |  |) /_ ,--.,--.|  ||  | ,---.      ,--,--.,--,--,  ,-|  |    '  .--./ ,---. ,--.   ,--. ,---. " << std::endl;
    std::cout << " |  .-.  \\|  ||  ||  ||  |(  .-'     ' ,-.  ||      \\' .-. |    |  |    | .-. ||  |.'.|  |(  .-' " << std::endl;
    std::cout << " |  '--' /'  ''  '|  ||  |.-'  `)    \\ '-'  ||  ||  |\\ `-' |    '  '--'\\' '-' '|   .'.   |.-'  `) " << std::endl;
    std::cout << " `------'  `----' `--'`--'`----'      `--`--'`--''--' `---'      `-----' `---' '--'   '--'`----' " << std::endl;
    std::cout << "              /)  (\\             " << std::endl;
    std::cout << "         .~._((,==.))_.~,                                               .~.___,==.___.~,        " << std::endl;
    std::cout << "           `~.   OO   ,~'                                                 `~.   OO   ,~'        " << std::endl;
    std::cout << "             / ,o~~o. \\                                                     / ,o~~o. \\     " << std::endl;
    std::cout << "            { { .__. } }                                                   { { .__. } }      " << std::endl;
    std::cout << "             ) `~~~\\' (                                                     ) `~~~\\' (          " << std::endl;
    std::cout << "            /`-._  _\\.-\\                                                   /`-._  _\\.-\\        " << std::endl;
    std::cout << "           /         )  \\                                                 /         )  \\        " << std::endl;
    std::cout << "         ,-X        #   X-.                                              ,-X        #   X-.       " << std::endl;
    std::cout << "        /   \\         /   \\                                             /   \\         /   \\     " << std::endl;
    std::cout << "       (    )| |  | |(     )                                           (     )| |  | |(    )      " << std::endl;
    std::cout << "        \\   / | |  | | \\   /                                            \\   / | |  | | \\   /    " << std::endl;
    std::cout << "         \\_(.-( )--( )-.)_/                                              \\_(.-( )--( )-.)_/      " << std::endl;
    std::cout << "         /_,\\ ) /  \\ ( /._\\                                              /_,\\ ) /  \\ ( /._\\    " << std::endl;
    std::cout << "             /_,\\  /._\\                                                      /_,\\  /._\\         " << std::endl;
    
    // Intro and Rules
    std::cout << "\n\n" << "Welcome to Bulls and Cows, a fun word game\n";
    std::cout << "Can you guess the isogram (word with no repeated letters) I'm thinking of?\n\n";
    std::cout << "Bulls: The right letter in the right place. \nCows: The right letter in the wrong place. \n\n";

    return;
}

void InputWordLength()
{
    FText WordLength;
    std::cout << "What word length would you like? (3-7 letters): " << std::endl;
    std::getline(std::cin, WordLength);
    BCGame.GetWord(WordLength);
    return;
}

void InputDifficulty()
{
    FText Difficulty;
    std::cout << std::endl << "Choose a difficulty (easy/normal/hard): " << std::endl;
    std::getline(std::cin, Difficulty);
    BCGame.SetDifficulty(Difficulty);
    return;
}

FText GetValidGuess()
{
    FText Guess = "";
    EWordStatus Status = EWordStatus::Invalid_Status;
    do
    {
        std::cout << std::endl << "Try " << BCGame.GetCurrentTry() << " of " << BCGame.GetMaxTries() << ". Enter your guess: ";
        std::getline(std::cin, Guess);
        
        Status = BCGame.CheckGuessValidity(Guess);
        switch(Status)
        {
            case EWordStatus::Wrong_Length:
                std::cout << "Please enter a " << BCGame.GetHiddenWordLength() << " letter word.\n";
                break;
            case EWordStatus::Not_Lowercase:
                std::cout << "Please enter your guess in lowercase.\n";
                break;
            case EWordStatus::Not_Isogram:
                std::cout << "Please enter an isogram (no repeated letters).\n";
                break;
            default:
                break;
        }
    } while(Status != EWordStatus::OK);
    return Guess;
}

void PlayGame() {
    BCGame.Reset();
    int32 MaxTries = BCGame.GetMaxTries();
    while(!BCGame.IsGameWon() && BCGame.GetCurrentTry() <= MaxTries)
    {
        FText Guess = GetValidGuess();
        
        FBullCowCount BullCowCount = BCGame.SubmitValidGuess(Guess);
        std::cout << "Bulls = " << BullCowCount.Bulls;
        std::cout << ". Cows = " << BullCowCount.Cows << "." <<std::endl;
    }
    PrintGameSummary();
    return;
}

bool AskToPlayAgain()
{
    bool PlayAgain;
    FText Response = "";
    std::cout << std::endl << "Would you like to play again with the same word? (yes/no)" << std::endl;
    std::getline(std::cin, Response);
    if (Response[0] == 'y' || Response[0] == 'Y')
    {
        PlayAgain = true;
        return PlayAgain;
    }
    else if (Response[0] == 'n' || Response[0] == 'N')
    {
        bool PlayAgain = false;
        return PlayAgain ;
    }
    else
    {
        std::cout << std::endl << "Please enter a valid answer." << std::endl;
        return AskToPlayAgain();
    }
}

//would the user like to play again but with a different word?
bool PlayAgainSameWord()
{
    bool SameWord;
    FText Response = "";
    std::cout << std::endl << "Would you like to play again with a different word? (yes/no)" << std::endl;
    std::getline(std::cin, Response);
    if (Response[0] == 'y' || Response[0] == 'Y')
    {
        SameWord = true;
        return SameWord;
    }
    else if (Response[0] == 'n' || Response[0] == 'N')
    {
        SameWord = false;
        return SameWord;
    }
    else
    {
           std::cout << std::endl << "Please enter a valid answer." << std::endl;
           return PlayAgainSameWord();
    }
}

//Print win condition
void PrintGameSummary()
{
    std::cout << std::endl;
    if(BCGame.IsGameWon())
    {
        std::cout << "WELL DONE - YOU WON! \n";
    }
    else
    {
        std::cout << "Better luck next time! \n";
    }
}
